"""
"""
from logging import getLogger

from naludaq.helpers import type_name
from naludaq.helpers.semiton import SemitonABC

from .default import BoardController

LOGGER = getLogger("naludaq.board_controller_default")


class BoardControllerAsocv3(SemitonABC, BoardController):
    def toggle_trigger(self, cycles: int = 3):
        """Toggle the trigger signal."""
        if not isinstance(cycles, int):
            raise TypeError(f'"cycles" must be an int, not {type_name(cycles)}')
        if cycles < 1:
            raise ValueError(f'"cycles" must be at least 1, got {cycles}')
        if cycles > 2**16 - 1:
            raise ValueError(
                f'"cycles" must be at most {2**16 - 1} (16-bits), got {cycles}'
            )

        LOGGER.debug("Sending software trigger")
        cmd = f"C000{cycles:04X}"
        self._send_command(cmd)



"""Helper functions for validating inputs."""
import pathlib


def validate_dir_or_raise(output_dir, name="Directory"):
    if not isinstance(output_dir, (str, pathlib.Path)):
        raise TypeError(f"{name} must be a string.")
    if not pathlib.Path(output_dir).exists():
        raise FileNotFoundError(f"{name} must exist.")
    if output_dir is None or not pathlib.Path(output_dir).is_dir():
        raise NotADirectoryError(f"{name} must be specified.")


def validate_chip_list_or_raise(chips: list[int], params: dict):
    num_chips = len(params.get("chips", ["single"]))
    if not isinstance(chips, list):
        raise TypeError("Chips must be a list.")
    if len(chips) != len(set(chips)):
        raise ValueError("One or more chip numbers is repeated.")
    for chip in chips:
        if not isinstance(chip, int):
            raise TypeError("Chips must be a list of integers.")
        if not 0 <= chip < num_chips:
            raise ValueError("One or more chip numbers is out of range.")
